#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..Eloquent.BaseBuilder import BaseBuilder
import pymysql

class Expression(object):

    def __init__(self, name=None):
        self.__name = name

    def format_column(self, key, model=None):
        if model is not None and model.columns and key in model.columns:
            return '`{}`'.format(key)
        return key

    def format_column_for_cassandra(self, key, model=None):
        if model is not None and model.columns and key in model.columns:
            return '"{}"'.format(key)
        return key

    def format_string(self, key):
        if isinstance(key, Expression):
            return key.__name
        elif isinstance(key, str):
            if key == "NULL" or key == "null":
                return "{}".format(key)
            return "'{}'".format(pymysql.converters.escape_string(key))
        elif isinstance(key, BaseBuilder):
            return "({})".format(key.tosql())
        return key

    def list_to_str(self, data):
        if data and isinstance(data, list):
            return tuple(data).__str__().replace(",)", ")") if len(data) == 1 else tuple(data).__str__()
        if data and isinstance(data, tuple):
            return data.__str__()
        elif isinstance(data, BaseBuilder):
            return "(" + data.tosql() + ")"
        else:
            raise Exception('param invalid')


expression = Expression()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .Connection import Connection
from ..Util.Logger import logger
from cassandra.cluster import Cluster
from cassandra.auth import PlainTextAuthProvider
from cassandra.cluster import NoHostAvailable
from cassandra.policies import RoundRobinPolicy
from cassandra.connection import ssl
import threading


class CassandraConnection(Connection):
    _connection = {}
    _instance = {}
    _lock = threading.Lock()

    def __init__(self, database, config):
        self._logger = None
        if config.get('LOG_DIR', None) is not None:
            self._logger = logger.set_path(config['LOG_DIR'])
        self._database = database
        self._config = config

    def execute(self, sql):
        self.log(sql)
        rows = self.connect().execute(sql)
        return [dict(row._asdict()) for row in rows]

    def connect(self):
        if self._connection.get(self._database) is None:
            with self._lock:
                if self._connection.get(self._database) is None:
                    auth_provider = PlainTextAuthProvider(
                        username=self._config['DB_USER'],
                        password=self._config['DB_PASSWORD']
                    )

                    ssl_options = None
                    if self._config.get('SSL_CA_CERTS'):
                        ssl_options = {
                            'ca_certs': self._config['SSL_CA_CERTS'],
                            'ssl_version': ssl.PROTOCOL_TLSv1_2,
                            'certfile': self._config.get('SSL_CERTFILE'),
                            'keyfile': self._config.get('SSL_KEYFILE')
                        }

                    try:
                        cluster = Cluster(
                            self._config['DB_HOST'].split(","),
                            port=int(self._config.get('DB_PORT', 9042)),
                            auth_provider=auth_provider,
                            load_balancing_policy=RoundRobinPolicy(),
                            ssl_options=ssl_options
                        )
                        session = cluster.connect()
                        session.set_keyspace(self._config['DB_KEYSPACE'])
                        self._connection[self._database] = session
                    except NoHostAvailable as e:
                        if self._logger:
                            self._logger.error(f"Could not connect to Cassandra: {e}")
                        raise
        return self._connection[self._database]

    @classmethod
    def instance(cls, database, config):
        if cls._instance.get(database) is None:
            with cls._lock:
                if cls._instance.get(database) is None:
                    cls._instance[database] = CassandraConnection(database, config)
        return cls._instance[database]


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pandas
import decimal

class Response(object):

    def __init__(self, data=None):
        self.__data__ = data

    def data(self):
        return pandas.DataFrame(self.__data__)

    def tolist(self, columns):
        if self.__data__ is not None:
            if type(columns) == str:
                return [self.format_decimal(index[columns]) for index in self.__data__]
            elif type(columns) == list:
                return [self._muti_select(index, columns) for index in self.__data__]
        return []

    def pluck(self, key, value):
        if self.__data__ is not None:
            return {index[key]: self.format_decimal(index[value]) for index in self.__data__}
        return {}

    def _muti_select(self, dict, columns):
        item = []
        for index in columns:
            item.append(self.format_decimal(dict[index]))
        return item

    def format_decimal(self, val):
        return str(val) if isinstance(val, decimal.Decimal) else val

    def get(self):
        return self.__data__

    def lists(self, columns):
        return self.tolist(columns)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""metaclass静态方法调用处理类"""

__author__ = ''


class BaseMagicMetaClass(type):
    def __getattr__(self, key):
        if key == "__new__":
            return object.__new__(self)
        try:
            return object.__getattribute__(self, key)
        except:
            return getattr(self.__new__(self), key)


class MagicMetaClass(metaclass=BaseMagicMetaClass):
    pass



import psycopg2.pool


class Connectionpool():
    _pool = {}

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Connectionpool, cls).__new__(cls, *args, **kwargs)
            return cls._instance

    def connection(self, pro_db_config, database):
        if self._pool.get(database, None) is None:
            self._pool[database] = psycopg2.pool.ThreadedConnectionPool(
                minconn=5,
                maxconn=50,
                **pro_db_config
            )

        return self._pool[database].getconn()


connectionpool = Connectionpool()
